@@ -87,6 +87,9 @@
 #'  e.g. Smoking; No. 25 observations, where there is a new line after the
 #'  factor name. If you want a different text for the second line you can
 #'  specifically use the \code{\link[base]{sprintf}} formatting, e.g. "No. \%s patients".
+#' @param missing_value Value that is substituted for empty cells. Defaults to "-"
+#' @param names_of_missing Optional character vector containing the names of returned statistics,
+#'  in case all returned values for a given \code{by} level are missing. Defaults to NULL
 #' @param ... Currently only used for generating warnings of deprecated call
 #'  parameters.
 #' @return Returns a vector if vars wasn't specified and it's a
@@ -126,6 +129,8 @@ getDescriptionStatsBy <- function(x,
                                   NEJMstyle = FALSE,
                                   percentage_sign = TRUE,
                                   header_count,
+                                  missing_value = "-",
+                                  names_of_missing = NULL,
                                   ...){
 
   API_changes <-
@@ -231,12 +236,15 @@ getDescriptionStatsBy <- function(x,
   # that one should be used otherwise go
   # with the name of the variable
   if (label(x) == "")
-    name <- deparse(substitute(x))
+    name <- paste0(deparse(substitute(x)), collapse = "")
   else
     name <- label(x)
 
-  if (is.logical(x))
+  if (is.logical(x)) {
     x <- factor(x, levels=c(TRUE, FALSE))
+  } else if (is.character(x) && any(table(x,by) == 0)) {
+    x <- factor(x)
+  }
 
   # Check missing -
   # Send a warning, since the user might be unaware of this
@@ -260,8 +268,8 @@ getDescriptionStatsBy <- function(x,
     useNA <- "always"
 
   # If all values are to be shown then simply use
-  # the factors function
-  if (show_all_values)
+  # the factors function, provided describeProp was specified
+  if (show_all_values & deparse(substitute(prop_fn)) == "describeProp")
     prop_fn <- describeFactors
 
   addEmptyValuesToMakeListCompatibleWithMatrix <- function(t){
@@ -295,7 +303,7 @@ getDescriptionStatsBy <- function(x,
     ret <- list()
     for (n in names(t)){
       # Create an empty array
-      ret[[n]] <- rep(0, times=length(all_row_names))
+      ret[[n]] <- rep(missing_value, times=length(all_row_names))
       names(ret[[n]]) <- all_row_names
       # Loop and add all the values
       for (nn in all_row_names){
@@ -329,6 +337,25 @@ getDescriptionStatsBy <- function(x,
               useNA.digits = useNA.digits,
               percentage_sign = percentage_sign)
 
+    missing_t <- sapply(t, is.null)
+    if (any(missing_t)) {
+      substitute_t <- rep(missing_value, length(t[!missing_t][[1]]))
+      names(substitute_t) <- names(t[!missing_t][[1]])
+      for (i in seq_along(t[missing_t])) {
+        t[missing_t][[i]] <- substitute_t
+      }
+    }
+    
+    if (all(unlist(sapply(t, is.na))) & !is.null(names_of_missing)) {
+      substitute_t <- rep(missing_value, length(names_of_missing))
+      names(substitute_t) <- names_of_missing
+      substitute_list <- vector("list", length = length(t))
+      names(substitute_list) <- names(t)
+      for (i in seq_along(substitute_list)) {
+        substitute_list[[i]] <- substitute_t
+      }
+      t <- substitute_list
+    }
 
     if (length(t[[1]]) != 1){
       fn_name <- deparse(substitute(continuous_fn))
@@ -355,6 +382,26 @@ getDescriptionStatsBy <- function(x,
             useNA.digits = useNA.digits,
             default_ref = default_ref, percentage_sign = percentage_sign)
 
+    missing_t <- sapply(t, is.null)
+    if (any(missing_t)) {
+      substitute_t <- rep(missing_value, length(t[!missing_t][[1]]))
+      names(substitute_t) <- names(t[!missing_t][[1]])
+      for (i in seq_along(t[missing_t])) {
+        t[missing_t][[i]] <- substitute_t
+      }
+    }
+
+    if (all(unlist(sapply(t, is.na))) & !is.null(names_of_missing)) {
+      substitute_t <- rep(missing_value, length(names_of_missing))
+      names(substitute_t) <- names_of_missing
+      substitute_list <- vector("list", length = length(t))
+      names(substitute_list) <- names(t)
+      for (i in seq_along(substitute_list)) {
+        substitute_list[[i]] <- substitute_t
+      }
+      t <- substitute_list
+    }
+    
     # Check that we're dealing with only one row
     if (unique(sapply(t, length)) == 1)
       # Set the rowname to a special format
@@ -379,7 +426,7 @@ getDescriptionStatsBy <- function(x,
 
     # If this is the only row then set that row to the current name
     if (length(t[[1]]) == 1){
-      names(t[[1]][1]) = name
+      names(t[[1]][1]) <- name
     }
 
   }else{
@@ -397,6 +444,25 @@ getDescriptionStatsBy <- function(x,
               useNA.digits = useNA.digits,
               percentage_sign = percentage_sign)
     }
+    missing_t <- sapply(t, is.null)
+    if (any(missing_t)) {
+      substitute_t <- rep(missing_value, length(t[!missing_t][[1]]))
+      names(substitute_t) <- names(t[!missing_t][[1]])
+      for (i in seq_along(t[missing_t])) {
+        t[missing_t][[i]] <- substitute_t
+      }
+    }
+    
+    if (all(unlist(sapply(t, is.na))) & !is.null(names_of_missing)) {
+      substitute_t <- rep(missing_value, length(names_of_missing))
+      names(substitute_t) <- names_of_missing
+      substitute_list <- vector("list", length = length(t))
+      names(substitute_list) <- names(t)
+      for (i in seq_along(substitute_list)) {
+        substitute_list[[i]] <- substitute_t
+      }
+      t <- substitute_list
+    }
   }
 
   # Convert the list into a matrix compatible format
